#include "BitLabelSet.h"


const  unsigned char CBitLabelSet::bitmask[8]= {0x01u,0x02u,0x04u,0x08u,0x10u,0x20u,0x40u,0x80u};
const  unsigned char CBitLabelSet::sizeBuf[256]={
0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3,
 4, 3, 4, 4, 5, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4
, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4,
 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 1, 2, 2, 3, 2, 3
, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5,
 6, 5, 6, 6, 7, 2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5
, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8};

CBitLabelSet ::CBitLabelSet(int number)
{
	length=  (number/8)+1;
	labels = (unsigned char *) calloc(length, sizeof(unsigned char));
	size=0;
}

CBitLabelSet ::CBitLabelSet()
{
	length= 0 ;
	labels = 0;
	size=0;

}

CBitLabelSet ::~CBitLabelSet()
{
	free(labels);
}

void CBitLabelSet :: AllocMem(int number)
{
	length=  (number/8)+1;
	labels = (unsigned char *) calloc(length, sizeof(unsigned char));
	size=0;

}

UNSIGNCHAR CBitLabelSet :: GetSize()
{
	return size;
}

void CBitLabelSet :: GetLabels(unsigned char  *out)
{
     memcpy(out,labels,length);
}

int CBitLabelSet ::In(UNSIGNCHAR label)
{

    UNSIGNCHAR location= (label-1)/8;
    UNSIGNCHAR bit=(label-1)%8;
    
  //printf("in %d,loc=%d, bit=%d, %x\n",label,location,bit, labels[location]);

    if (((labels[location]) & (bitmask[bit]))==0) 
    { 
        //printf("not in\n"); 
       return -1;
    }  
    else
    {
      return 1;
    }
}

//// add an element to the set. and size++
/// pls make sure no repeated element.
int CBitLabelSet ::Add(UNSIGNCHAR label)
{
    UNSIGNCHAR location= (label-1)/8;
    UNSIGNCHAR bit=(label-1)%8;
    
    //printf("before add %d,loc=%d, bit=%d, %x",label,location,bit, labels[location]);
    
    if (((labels[location]) & (bitmask[bit]))!=0) 
      return 0;
    else
      labels[location]=(labels[location]) | (bitmask[bit]); 
   
    size++;
  // printf("after add: size=%d,loc=%d, bit=%d, %x\n",size,location,bit, labels[location]);  
    return 1;
}

void CBitLabelSet::Clear()
{
    memset(labels,0,length);
    size=0;
}

unsigned char CBitLabelSet ::Pop(UNSIGNCHAR i)
{
    return labels[i];
}

int CBitLabelSet ::Equal(CBitLabelSet * labelSet1)
{
	UNSIGNCHAR i;
	
	if (size!=labelSet1->GetSize())
	  return 0;
	for(i=0;i<length;i++)
	  if (labels[i]!=labelSet1->Pop(i))
        return 0;
    return 1;

}

// check whether I am a subset of labelSet1 
int CBitLabelSet ::SubsetOf(CBitLabelSet * labelSet1)
{
	UNSIGNCHAR i, sizeLabelSet1;
    unsigned char label; 
   
    sizeLabelSet1=labelSet1->GetSize();
	if (size > sizeLabelSet1)  /// I am bigger, not a subset 
	  return 0;

	for(i=0;i<length;i++)
	{
        label=labelSet1->Pop(i);
        if ( (label&(labels[i]))!=labels[i])
          return 0;
    }     
	 return 1;

}

///// add labelSet1 to me
int CBitLabelSet ::Union(CBitLabelSet * labelSet1)
{
	UNSIGNCHAR i;
	unsigned char label;

    size=0;
    for(i=0;i<length;i++)
    {
        label=labelSet1->Pop(i);
        labels[i]= label|(labels[i]);
        size+=sizeBuf[labels[i]];
    }
    return size;

}



// substract labelSet1 from me
//// {1,3,4,5}-{3,4}={1,5}
int CBitLabelSet ::Substract(CBitLabelSet * labelSet1)
{
	UNSIGNCHAR i;
	unsigned char label;

    size=0;
    for(i=0;i<length;i++)
    {
        label=labelSet1->Pop(i);
        labels[i]= label&(~(labels[i]));
        size+=sizeBuf[labels[i]];
    }
    return size;
}



void CBitLabelSet::Display()
{
 
    printf("{");

	unsigned char i;
   
    for (i=1;i<=length*8;i++)
      if (In(i))
        printf("%d,",i);
    printf("}\n");
}




void CBitLabelSet ::Copy(CBitLabelSet *labelSet1)
{
	Clear();
	labelSet1->GetLabels(labels);
	size=labelSet1->GetSize();	

}




