#ifndef __QuartetTable_H
#define __QuartetTable_H

#include "general.h"
#include "BitLabelSet.h"

class CQuartetTable
{
private:

  //char quartets[MAXILABEL][MAXILABEL][MAXILABEL][MAXILABEL];
  UNSIGNCHAR ****quartets;


  int total;   ////total number of quartets stored in the table
  int labelNum;  /// total number of taxa
  char **names;  /// the list of names for taxa.
  float **distance; /// the distance matrix 

public:
  CQuartetTable(int number);
  CQuartetTable();

 ~CQuartetTable();

private:
  //// free the list whose head is headnode.
  void freelist(struct Quartet *headnode);
  void ClearConflictList();
  void exchange(UNSIGNCHAR *a, UNSIGNCHAR *b);
  void insert_list(struct Quartet *node);

///sort all the quartets and insert them into a list.
/// Currently, only insert function is implemented.
  void sort();
  
  
  void compare(UNSIGNCHAR a, UNSIGNCHAR b, UNSIGNCHAR *small, UNSIGNCHAR *big);
  
  
  int AddLabel(UNSIGNCHAR label, CBitLabelSet *labelSet, int *edges, int lca, int rootLabel);


public:
   void SetLabelNum(UNSIGNCHAR number);
   UNSIGNCHAR GetValue(UNSIGNCHAR i, UNSIGNCHAR j, UNSIGNCHAR k, UNSIGNCHAR l);
   int LoadFromFile(char *filename);
   void save(char *filename);
   int Compare(CQuartetTable *source);
   void Update(UNSIGNCHAR *change_history);
   
   void ToBracketTree();
   void LoadFromDistanceMatrix(char * filename);
   void GenerateQuartet_FourPoint();
   int  Insert(UNSIGNCHAR q[4], UNSIGNCHAR weight);
   

   void CheckChangeHistory(UNSIGNCHAR *change_history,int error_num);
   
   char * GetLabelName(UNSIGNCHAR label);

   void ClearTable();
   int getsize();


   void IncreaseWeight(UNSIGNCHAR *quartet, UNSIGNCHAR weight) ;

  ///for a given four taxa of a quartet, find whether there is a same quartet that
  ///has been loaded.
  ///if the four taxa labels are same(don't care their order), then they are same.
  /// return: 0: the quartet doesn't exist in hash table
  ///         1: the quartet exists in hash table and have same structure with query quartet
  ///         2: the quartet exists in hash talbe but they have differernt structure.
   int find(UNSIGNCHAR *quartet);
   int find(UNSIGNCHAR *quartet, UNSIGNCHAR *conflict);
   int find(UNSIGNCHAR a, UNSIGNCHAR b,UNSIGNCHAR c, UNSIGNCHAR d, UNSIGNCHAR *conflict);

   void Copy(CQuartetTable *source);
   void ChangeQuartet(UNSIGNCHAR old_q[4],UNSIGNCHAR new_q[4]);


 ///for a given four taxa of a quartet, find whether there is a same quartet that
  ///has been loaded.
  ///if the four taxa labels are same(don't care their order), then they are same.
  /// return: 0: the quartet doesn't exist in hash table
  ///         1: the quartet exists in hash table and have same structure with query quartet
  ///         2: the quartet exists in hash table but they have differernt structure.
  ///            the quartet in the hash table is returned in conflict

  ///  if the quartet exists in the hash table, we increase the confidence of that quartet
  ///  with the value stored in confidence.
  ///  if we don't want to change the weight value, just put 0 as that argument.
  ///  if we want increase the weight by one each time we find the same quaretet,
  //// put that argument as 1.
  int find(UNSIGNCHAR *quartet, UNSIGNCHAR *conflict,UNSIGNCHAR weight);

  /// disply all the quartet in the list.
  void display();

  /////save the quartet into file
  void save(char *filename, char IncludeConflict);

 

      /// given four labels of a quartet ab|cd, format them to a standard quartet.
      /// that is  a<b, c<d, and a<c. save the formatted quartet to quartet[4]
  void formquartet(UNSIGNCHAR a,UNSIGNCHAR b,UNSIGNCHAR c,UNSIGNCHAR d,UNSIGNCHAR *quartet);

 } ;
 #endif
 ;



